Package: countsimQC
Type: Package
Title: Compare Characteristic Features of Count Data Sets
Version: 1.31.0
Authors@R: c(person("Charlotte", "Soneson", role = c("aut", "cre"), 
    email = "charlottesoneson@gmail.com", comment = c(ORCID = "0000-0003-3833-2169")))
Description: countsimQC provides functionality to create a comprehensive report 
    comparing a broad range of characteristics across a collection of count 
    matrices. One important use case is the comparison of one or more synthetic 
    count matrices to a real count matrix, possibly the one underlying the 
    simulations. However, any collection of count matrices can be compared.
License: GPL (>=2)
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: rmarkdown (>= 2.5), edgeR, DESeq2 (>= 1.16.0), dplyr, tidyr,
        ggplot2, grDevices, tools, SummarizedExperiment, genefilter,
        DT, GenomeInfoDbData, caTools, randtests, stats, utils,
        methods, ragg, rlang
RoxygenNote: 7.3.3
Suggests: knitr, testthat
VignetteBuilder: knitr
biocViews: Microbiome, RNASeq, SingleCell, ExperimentalDesign,
        QualityControl, ReportWriting, Visualization, ImmunoOncology
URL: https://github.com/csoneson/countsimQC
BugReports: https://github.com/csoneson/countsimQC/issues
git_url: https://git.bioconductor.org/packages/countsimQC
git_branch: devel
git_last_commit: 7740a95
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:03:05 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>)
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>
