# Generated by roxygen2: do not edit by hand

export(countsimQCReport)
export(generateIndividualPlots)
import(DESeq2)
import(DT)
import(GenomeInfoDbData)
import(SummarizedExperiment)
import(dplyr)
import(edgeR)
import(genefilter)
import(ggplot2)
import(ragg)
import(tidyr)
importFrom(caTools,trapz)
importFrom(edgeR,cpm)
importFrom(genefilter,rowVars)
importFrom(ggplot2,is_ggplot)
importFrom(grDevices,n2mfrow)
importFrom(methods,is)
importFrom(randtests,runs.test)
importFrom(rlang,.data)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_exec)
importFrom(rmarkdown,render)
importFrom(stats,chisq.test)
importFrom(stats,cor)
importFrom(stats,ecdf)
importFrom(stats,ks.test)
importFrom(stats,model.matrix)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,combn)
